#ifndef PURECWRAPPER_HH
#define PURECWRAPPER_HH

#include "BPNGDefines.h"
#include "IBPNGClient.h"
#include "IBPNGClientListener.h"
#include "IClientProperties.h"

extern "C"
{
    struct BPNGClientListenerFunctionPointers
    {
        void (WINAPI* onBPNGDeviceDetected)(OnlineLoggerInfo* info);
        void (WINAPI* onBPNGDeviceDisappeared)(OnlineLoggerInfo* info);
        void (WINAPI* onBPNGDeviceStateChange)(OnlineLoggerInfo* info);
        int (WINAPI* onProgressDataDownload)(int percentCompleted);
        int (WINAPI* onProgressConversion)(int percentCompleted, const char* status);
        void (WINAPI* onStatusMessage)(const char* statusMsg);
        int (WINAPI* onDataRecoverProgress)(const char* statusMsg, int percentage);
        void (WINAPI* onWarning)(BPNGWarningCode warningCode, const char* warnMsg);
        int (WINAPI* onTargetPathTooLong)(char* newTarget, int maxSize);
        int (WINAPI* getOverwritingPermission)(const char* filePath);
        const char* (WINAPI* onLogInDataRequired)(unsigned mbnr);
        void (WINAPI* onInvalidPwConfigFound)(unsigned mbnr);
        void (WINAPI* onLogInDataFailed)();
        void (WINAPI* onResetLogInDataFailed)();
        void (WINAPI* onFuncAccessDenied)();
        int (WINAPI* onCriticalDiskSpace)(uint64_t freeSpace, uint64_t neededSpace, const char* drive, const char* msg);
        void (WINAPI* onFirmwareUpdateProgress)(int progress, int stepId, int subStepId, const char* desc);
        void (WINAPI* onFirmwareUpdateError)(int errorId);
        int (WINAPI* onGetLogReportProgress)(int progress, const char* desc);
        void (WINAPI* onDownloadStart)(int64_t totalAmountOfBytes);
        void (WINAPI* onConversionStart)(int64_t totalAmountOfBytes);
        const char* (WINAPI* onExtractionPasswordRequired)(unsigned int retryCount);
        int (WINAPI* onProgressDeletion)(int percentCompleted);
        int (WINAPI* isTerminateLiveDownloadRequest)();
        int (WINAPI* onProgressDataDownload2)(int percentCompleted, uint64_t downloadedSize, uint64_t totalsize);
    };

    DECLDIR void WINAPI setDevice(IBPNGClient* clientPtr, OnlineLoggerInfo* device);
    DECLDIR void WINAPI setTSLCluster(IBPNGClient* clientPtr, int num, OnlineLoggerInfo* loggerArray);
    DECLDIR BOOL WINAPI setOfflineData(IBPNGClient* clientPtr, const char* path);

    DECLDIR BOOL WINAPI initialize(IBPNGClient* clientPtr);
    DECLDIR int WINAPI downloadDataSpans(IBPNGClient* clientPtr, uint16_t numSpans, DataSpan* dataSpans, const char* target, BOOL doSorting);
    DECLDIR int WINAPI startLiveDownload(IBPNGClient* clientPtr, uint64_t startTimeStamp, const char* target);
    DECLDIR int WINAPI convertData(IBPNGClient* clientPtr, IConversionSet* conversionSet, const char* target);
    DECLDIR void WINAPI scanNetworkForLogger(IBPNGClient* clientPtr);
    DECLDIR void WINAPI activateGatewayLoggerDetection(IBPNGClient* clientPtr);
    DECLDIR IRdbEventList* WINAPI getEventList(IBPNGClient* clientPtr);
    DECLDIR BOOL WINAPI synchronizeRdb(IBPNGClient* clientPtr);
    DECLDIR const char* WINAPI getInstanceName(IBPNGClient* clientPtr);
    DECLDIR int WINAPI getInstanceId(IBPNGClient* clientPtr);
    DECLDIR const char* WINAPI getReferenceDataBasePath(IBPNGClient* clientPtr);
    DECLDIR const char* WINAPI getConfigPath(IBPNGClient* clientPtr);
    DECLDIR const char* WINAPI getDeviceName(IBPNGClient* clientPtr);
    DECLDIR BOOL  WINAPI getConfig(IBPNGClient* clientPtr, const char* path);
    DECLDIR BOOL  WINAPI reconfigLogger(IBPNGClient* clientPtr, int numLogger, OnlineLoggerInfoStringPair *loggerConfigPathPairs, const char* xsdVersionOfConfig);
    DECLDIR BOOL  WINAPI setDefaultConfig(IBPNGClient* clientPtr);
    DECLDIR int WINAPI deleteSectionsByStartUpIds(IBPNGClient* clientPtr, uint16_t numStartUpIds, uint64_t* startupIds);
    DECLDIR int WINAPI deleteAllData(IBPNGClient* clientPtr);
    DECLDIR BOOL WINAPI connectLogger(IBPNGClient* clientPtr);
    DECLDIR void WINAPI disconnectLogger(IBPNGClient* clientPtr);
    DECLDIR BOOL WINAPI isConnected(IBPNGClient* clientPtr);
    DECLDIR BOOL WINAPI setInfoEvent(IBPNGClient* clientPtr, const char* msg);
    DECLDIR BOOL WINAPI setMarker(IBPNGClient* clientPtr);
    DECLDIR int WINAPI getCurrentLoggerTime(IBPNGClient* clientPtr);
    DECLDIR int WINAPI setTime(IBPNGClient* clientPtr, int time);
    DECLDIR void WINAPI keepLoggerAlive(IBPNGClient* clientPtr, const char* ip);
    DECLDIR void WINAPI stopKeepLoggerAlive(IBPNGClient* clientPtr, const char* ip);
    DECLDIR OnlineLoggerInfo* WINAPI getOnlineLoggerInfo(IBPNGClient* clientPtr, const char* ip);
    DECLDIR BOOL WINAPI updateFirmware(IBPNGClient* clientPtr, OnlineLoggerInfoStringPair *loggerFirmwareUpdatePacketPair, BOOL force);
    DECLDIR BOOL WINAPI updatePBFirmware(IBPNGClient* clientPtr, OnlineLoggerInfoStringPair *loggerFirmwareUpdatePacketPair, BOOL force);
    DECLDIR const char* WINAPI getInternalPBVersion(IBPNGClient* clientPtr, OnlineLoggerInfoStringPair *loggerFirmwareUpdatePacketPair);
    DECLDIR BOOL WINAPI isUserAuthentificated(IBPNGClient* clientPtr, PwdPrivilegesFuncId actionName);
    DECLDIR BOOL WINAPI getVersions(IBPNGClient* clientPtr, OnlineLoggerInfoStringPair *versionPairs);
    DECLDIR const char* WINAPI getTMTVersion(IBPNGClient* clientPtr);
    DECLDIR BOOL WINAPI updateLicenses(IBPNGClient* clientPtr, OnlineLoggerInfoStringPair *loggerLicenseFilePair);
    DECLDIR const char* WINAPI getLicenses(IBPNGClient* clientPtr, unsigned deviceMbnr);
    DECLDIR BOOL WINAPI removeAllLicenses(IBPNGClient* clientPtr);

    DECLDIR const IChannelList* WINAPI getLoggerChannels(IBPNGClient* clientPtr);
    DECLDIR int WINAPI channelListGetSize(IChannelList* channelList);
    DECLDIR const IChannel* WINAPI channelListGetChannel(IChannelList* channelList, int index);
    DECLDIR ChannelType WINAPI channelGetType(IChannel* channel);
    DECLDIR int WINAPI channelGetIndex(IChannel* channel);
    DECLDIR const char* WINAPI channelGetName(IChannel* channel);
    DECLDIR int WINAPI channelGetMainboardNumber(IChannel* channel);
    DECLDIR int WINAPI channelGetOffset(IChannel* channel);
    DECLDIR BOOL WINAPI channelGetMappingActive(IChannel* channel);
    DECLDIR int WINAPI channelGetMappedChannelIndex(IChannel* channel);

    DECLDIR IFormatList* WINAPI getAvailableFormats(IBPNGClient* clientPtr);
    DECLDIR const int WINAPI formatsListGetSize(IFormatList* formats);
    DECLDIR const IFormatInfo* WINAPI formatListGetFormat(IFormatList* formats, int index);
    DECLDIR const char* WINAPI formatGetId(IFormatInfo* format);
    DECLDIR const char* WINAPI formatGetName(IFormatInfo* format, const char* languageCode);
    DECLDIR BOOL WINAPI formatIsMultiChannelSupport(IFormatInfo* format);
    DECLDIR BOOL WINAPI formatIsBinary(IFormatInfo* format);
    DECLDIR const char* WINAPI formatGetExtension(IFormatInfo* format);
    DECLDIR const char* WINAPI formatGetLicense(IFormatInfo* format);
    DECLDIR int WINAPI formatGetNumSupportedChannelTypes(IFormatInfo* format);
    DECLDIR ChannelType WINAPI formatGetChannelType(IFormatInfo* format, int index);

    DECLDIR size_t WINAPI eventListGetSize(IRdbEventList* eventList);
    DECLDIR const IRdbEvent* WINAPI eventListGetEvent(IRdbEventList* eventList, size_t index);

    DECLDIR RdbEventType WINAPI rdbEventGetType(IRdbEvent* rdbEvent);
    DECLDIR uint64_t WINAPI rdbEventGetUniqueId(IRdbEvent* rdbEvent);
    DECLDIR uint64_t WINAPI rdbEventGetTimeStamp(IRdbEvent* rdbEvent);
    DECLDIR const char* WINAPI rdbEventGetTimeZone(IRdbEvent* rdbEvent);
    DECLDIR int WINAPI rdbEventGetIndex(IRdbEvent* rdbEvent);
    DECLDIR const char* WINAPI rdbEventGetComment(IRdbEvent* rdbEvent);

    DECLDIR void WINAPI flashDeviceLED(IBPNGClient* clientPtr);
    DECLDIR int WINAPI createCCPXCPSeqFile(IBPNGClient* clientPtr, const char* xsdDir, const char* xmlDir, bool forceFlag);
    DECLDIR int WINAPI createCCPXCPDbcFiles(IBPNGClient* clientPtr, const char* dbcDir, const char* xsdDir, const char* xmlDir);
    DECLDIR const IFalseMeasureSignalList* WINAPI getFalseMeasureSignals(IBPNGClient* clientPtr);

    DECLDIR void WINAPI addListener(IBPNGClient* clientPtr, BPNGClientListenerFunctionPointers* pureCListener);
    DECLDIR void WINAPI removeListener(IBPNGClient* clientPtr, BPNGClientListenerFunctionPointers* pureCListener);

    DECLDIR BPNGError WINAPI getLastError(IBPNGClient* clientPtr);
    DECLDIR int WINAPI getNumConversionErrors(IBPNGClient* clientPtr);
    DECLDIR BPNGError WINAPI getConversionError(IBPNGClient* clientPtr, int index);
    DECLDIR int WINAPI getNumDownloadErrors(IBPNGClient* clientPtr);
    DECLDIR BPNGError WINAPI getDownloadError(IBPNGClient* clientPtr, int index);
    DECLDIR const char* WINAPI getFWVersion(IBPNGClient* clientPtr);
    DECLDIR int WINAPI resetMarkerCounter(IBPNGClient* clientPtr);
    DECLDIR int WINAPI downloadBugReport(IBPNGClient* clientPtr, const char* targetPath, BPNGBugreportMode mode, uint64_t startTime, uint64_t endTime);
    DECLDIR int WINAPI restartDevice(IBPNGClient* clientPtr, BOOL waitForRestart);
    DECLDIR int WINAPI shutdownDevice(IBPNGClient* clientPtr);
    DECLDIR void WINAPI release(IBPNGClient* clientPtr);
    DECLDIR BOOL WINAPI getMemoryFillLevel(IBPNGClient* clientPtr, MemoryFillLevel* fillLevel);

    DECLDIR int WINAPI setPwdFile(IBPNGClient* clientPtr, const char* path, unsigned targetMbnr);
    DECLDIR const char* WINAPI getPwdFile(IBPNGClient* clientPtr, unsigned sourceMbnr);
    DECLDIR BOOL WINAPI isPasswordProtectionSupported(IBPNGClient* clientPtr, unsigned deviceMbnr);

    DECLDIR BOOL WINAPI filterSignals(IBPNGClient* clientPtr, const char* pathToFilterSettings, const char* targetPath);
    DECLDIR int WINAPI createTestReport(IBPNGClient* clientPtr, IConversionSet* conversionSet, const char* target);


    DECLDIR void WINAPI setClientProperties(IBPNGClient* clientPtr, IClientProperties* properties);
    DECLDIR IClientProperties* WINAPI getClientProperties(IBPNGClient* clientPtr);
    DECLDIR void WINAPI saveProperties(IBPNGClient* clientPtr, const char* pathToIniFile);

    DECLDIR void WINAPI clearDatabaseFileAssignments(IBPNGClient* clientPtr);
    DECLDIR void WINAPI assignDatabaseFile(IBPNGClient* clientPtr, ChannelType channelType, int channelIndex, const char* databaseFilePath);

    DECLDIR void WINAPI addChannelByType(IConversionSet* conversionSet, ChannelType channelType, uint8_t channelIndex, const char* formatId, int fileId, int offset, int mainboardnumber, bool mappingActive, int mappingChannelId);
    DECLDIR void WINAPI addTimeSpan(IConversionSet* conversionSet, uint64_t startTime, uint64_t endTime, uint64_t id);
    DECLDIR void WINAPI addRdbIdRange(IConversionSet* conversionSet, uint64_t startId, uint64_t endId);
    DECLDIR bool WINAPI loadFormats(IConversionSet* conversionSet, const char* path);
    DECLDIR bool WINAPI saveFormats(IConversionSet* conversionSet, const char* path);

    DECLDIR const char* WINAPI encryptData(const char* data, uint64_t key);
    DECLDIR const char* WINAPI decryptData(const char* data, uint64_t key);

    DECLDIR void WINAPI setCommonProperties(IClientProperties* properties, const char* nameOfTester, uint32_t maxOutputSizeMB, BOOL separatedTimeFormat, BOOL separatedTimeFormatInOfflineSet,
        const char* alternativeLoggerName, BOOL useAlternativeLoggerName, BOOL useSubDirectories, BOOL midnightSplitting, BOOL fileTimeSpansLikeSelection,
        BOOL markerNumberInFileNames, BOOL subfolderWithLoggerName, int maxOfflineZipSizeMB, int maxOutputSizeMBSortedDownload,
        BOOL traceCutterStorage, const char* traceCutterFNPatternConversion, BOOL createOfflineDataOnTraceCutterStorage, const char* traceCutterFNPatternOfflineData,
        BOOL traceCutterMarkerCompact, BOOL pauseDataRecordingDuringDownload, BOOL tmASCIITimestampPrecisionToMicros, BOOL convertPhyStatusWithoutData);

    DECLDIR void WINAPI setUse10GLinkSpeed(IClientProperties* properties, BOOL use10GSpeedLink);

    DECLDIR const char* WINAPI getNameOfTester(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getMaxOutputSize(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getMaxOutputSizeSortedDownload(IClientProperties* properties);
    DECLDIR BOOL WINAPI isSeparatedTimeFormat(IClientProperties* properties);
    DECLDIR BOOL WINAPI isSeparatedTimeFormatInOfflineSet(IClientProperties* properties);
    DECLDIR const char* WINAPI getAlternativeLoggerName(IClientProperties* properties);
    DECLDIR BOOL WINAPI isAlternativeLoggerNameActive(IClientProperties* properties);
    DECLDIR BOOL WINAPI isConvertedFilesInSubDirsActive(IClientProperties* properties);
    DECLDIR BOOL WINAPI isMidnightSplittingActive(IClientProperties* properties);
    DECLDIR BOOL WINAPI isFileTimeSpansLikeSelection(IClientProperties* properties);
    DECLDIR BOOL WINAPI isMarkerNumbersInFileNames(IClientProperties* properties);
    DECLDIR BOOL WINAPI isSubfolderWithLoggerName(IClientProperties* properties);
    DECLDIR void WINAPI setCANPseudoMsgTimeStampProperties(IClientProperties* properties, BOOL writeTimeStampMsg, uint32_t channelIndex, uint32_t dlc, uint32_t canID,
        uint32_t hourBitPos, uint32_t minBitPos, uint32_t secBitPos, uint32_t dayBitPos, uint32_t monthBitPos, uint32_t yearBitPos);
    DECLDIR BOOL WINAPI isCANPseudoMsgTimeStampActive(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgChannelIndexTimeStamp(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgDlcTimeStamp(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgCanIDTimeStamp(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgHourBitPos(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgMinBitPos(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgSecBitPos(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgDayBitPos(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgMonthBitPos(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgYearBitPos(IClientProperties* properties);
    DECLDIR void WINAPI setCANPseudoMsgTriggerProperties(IClientProperties* properties, BOOL writeTriggerMessage, uint32_t channelIndex, uint32_t dlc, uint32_t canID, uint32_t triggerNumBitPos);
    DECLDIR BOOL WINAPI isCANPseudoMsgTriggerActive(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgChannelIndexTrigger(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgDlcTrigger(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgCanIDTrigger(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getCANPseudoMsgTriggerNumBitPos(IClientProperties* properties);
    DECLDIR void WINAPI setMOSTPseudoMsgProperties(IClientProperties* properties, BOOL active, uint32_t src, uint32_t target, uint32_t fktBlockID, uint32_t fktID);
    DECLDIR BOOL WINAPI isMOSTPseudoMsgActive(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getMOSTPseudoMsgSourceAddr(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getMOSTPseudoMsgTargetAddr(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getMOSTPseudoMsgFktBlockID(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getMOSTPseudoMsgFktID(IClientProperties* properties);
    DECLDIR void WINAPI setFlexRayPseudoMsgProperties(IClientProperties* properties, BOOL active, uint8_t channel, uint32_t slotID, uint32_t cycleCount, BOOL useFlexRayTypeDynamic);
    DECLDIR BOOL WINAPI isFlexRayPseudoMsgActive(IClientProperties* properties);
    DECLDIR uint8_t WINAPI getFlexRayPseudoMsgChannel(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getFlexRayPseudoSlotID(IClientProperties* properties);
    DECLDIR uint32_t WINAPI getFlexRayPseudoCycleCount(IClientProperties* properties);
    DECLDIR BOOL WINAPI isFlexRayPseudoMsgDynamicType(IClientProperties* properties);
    DECLDIR void WINAPI useSatelliteTimeForGPSFormats(IClientProperties* properties, BOOL flag);
    DECLDIR BOOL WINAPI isSatelliteTimeForGPSFormats(IClientProperties* properties);
    DECLDIR void WINAPI setIsochronousMost150Channels(IClientProperties* properties, const char* channels);
    DECLDIR const char* WINAPI getIsochronousMost150Channels(IClientProperties* properties);
    DECLDIR void WINAPI setAnalogToCANPseudoActive(IClientProperties* properties, BOOL flag);
    DECLDIR void WINAPI addAnalogPortSettings(IClientProperties* properties, uint16_t analogPort, BOOL isActive, uint32_t canChannel, uint32_t canID, const char* dbcPath);
    DECLDIR void WINAPI clearAnalogPortSettings(IClientProperties* properties);
    DECLDIR void WINAPI setDigitalToCANPseudoActive(IClientProperties* properties, BOOL flag);
    DECLDIR void WINAPI addDigitalPortSettings(IClientProperties* properties, uint16_t digitalPort, BOOL isActive, uint32_t canChannel, uint32_t canID, BOOL isExt);
    DECLDIR void WINAPI clearDigitalPortSettings(IClientProperties* properties);

    DECLDIR void WINAPI enableClientLogOutput(IBPNGClient* clientPtr);
}
#endif
