//************************************************************
//
// BPNGLoggerDetector.cc
//
/** @file BPNGLoggerDetector.cc
 *
 * @brief Description
 *
 * Description
 */
//************************************************************

#include <ctime>
#include <map>
#include <string.h>

#ifndef _MSC_VER
#include <tmlib-config.h>
#endif

#include "BPNGLoggerDetector.hh"

using namespace std;


// Returns a vector of BP2Devices, search the network for searchTimeOut seconds
vector<OnlineLoggerInfo> BPNGLoggerDetector::getLoggerList(unsigned searchTimeOut)
{
    m_loggerList.clear();

    IBPNGClient *client = getBPNGClient();

    client->addListener(this);

    time_t start = time(nullptr);

    if (searchTimeOut == 0)
    {
        // scan only once
        client->scanNetworkForLogger();
    }
    else
    {
        while (time(nullptr) - start < searchTimeOut)
        {
            client->scanNetworkForLogger();
        }
    }

    client->release();
    return m_loggerList;
}

vector<TSLClusterImpl> BPNGLoggerDetector::getTSLs(vector<OnlineLoggerInfo> loggersInNetwork)
{
    map<int, TSLClusterImpl> tsls;

    for (size_t i = 0; i < loggersInNetwork.size(); ++i)
    {
        OnlineLoggerInfo device = loggersInNetwork[i];
        if (device.tslNetworkId != -1)
        {
            auto id = tsls.find(device.tslNetworkId);
            if (id == tsls.end())
            {
                tsls.emplace(device.tslNetworkId, TSLClusterImpl(device));
            }
            else
            {
                id->second.addDevice(device);
            }
        }
    }

    vector<TSLClusterImpl> retVal;

    map<int, TSLClusterImpl>::iterator iter;
    for (iter = tsls.begin(); iter != tsls.end(); ++iter)
    {
        retVal.push_back(iter->second);
    }

    return retVal;
}

// listener function for observing blue PiraT 2 devices in network
void WINAPI BPNGLoggerDetector::onBPNGDeviceDetected(OnlineLoggerInfo *info)
{
    OnlineLoggerInfo newDevice = createCopy(info);

    bool alreadyIn = false;
    for (size_t i = 0; i < m_loggerList.size(); ++i)
    {
        if (m_loggerList[i].mbnr == newDevice.mbnr)
        {
            alreadyIn = true;
        }
    }

    if (!alreadyIn)
    {
        m_loggerList.push_back(newDevice);
    }
}

void WINAPI BPNGLoggerDetector::onBPNGDeviceDisappeared(OnlineLoggerInfo *info)
{
    OnlineLoggerInfo device = createCopy(info);
    for (size_t i = 0; i < m_loggerList.size(); ++i)
    {
        if (m_loggerList[i].mbnr == device.mbnr)
        {
            freeCopy(m_loggerList[i]);
            m_loggerList.erase(m_loggerList.begin() + i);
            break;
        }
    }
}

void WINAPI BPNGLoggerDetector::onBPNGDeviceStateChange(OnlineLoggerInfo *info)
{
    OnlineLoggerInfo device = createCopy(info);
    for (size_t i = 0; i < m_loggerList.size(); ++i)
    {
        if (m_loggerList[i].mbnr == device.mbnr)
        {
            m_loggerList[i] = device;
            break;
        }
    }
}

const char* getConstCharCopy(const char* what)
{
    size_t len = strlen(what) + 1;
    char* ret = (char*)malloc(len);
    strcpy(ret, what);
    return ret;
}

OnlineLoggerInfo BPNGLoggerDetector::createCopy(OnlineLoggerInfo *from)
{
    OnlineLoggerInfo copy;
    copy.ip = getConstCharCopy(from->ip);
    copy.name = getConstCharCopy(from->name);
    copy.mbnr = getConstCharCopy(from->mbnr);
    copy.deviceSN = getConstCharCopy(from->deviceSN);
    copy.occupied = from->occupied;
    copy.currentUser = getConstCharCopy(from->currentUser);
    copy.loggerStatus = from->loggerStatus;
    copy.wlan = from->wlan;
    copy.tslEth0IP = getConstCharCopy(from->tslEth0IP);
    copy.tslEth1IP = getConstCharCopy(from->tslEth1IP);
    copy.tslId = from->tslId;
    copy.tslNetworkId = from->tslNetworkId;
    copy.tslName = getConstCharCopy(from->tslName);
    copy.deviceType = from->deviceType;
    copy.fwVersion = getConstCharCopy(from->fwVersion);
    copy.tmpBusPort = from->tmpBusPort;
    copy.udpPort = from->udpPort;
    copy.ftpPort = from->ftpPort;
    copy.sfpIp = getConstCharCopy(from->sfpIp);
    copy.sfpPort = from->sfpPort;

    return copy;
}

void BPNGLoggerDetector::freeCopy(OnlineLoggerInfo who)
{
    free((char*)who.ip);
    free((char*)who.name);
    free((char*)who.mbnr);
    free((char*)who.deviceSN);
    free((char*)who.currentUser);
    free((char*)who.tslEth0IP);
    free((char*)who.tslEth1IP);
    free((char*)who.tslName);
    free((char*)who.fwVersion);
    free((char*)who.sfpIp);
}
