/*
Sample project for BPNGOnlineStreamingLib (blue PiraT 2 Onlinestreaming Library)
*/
#include <iostream>
#ifdef _MSC_VER
#include <conio.h>
#else
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#endif

#include "DataStreamingListener.h"
#include "IBPNGStreaming.h"
#include "BPNGDefines.h"

#include <time.h>

using namespace std;


#ifndef _MSC_VER
int _getch() {
    struct termios oldt, newt;
    int ch;
    tcgetattr( STDIN_FILENO, &oldt );
    newt = oldt;
    newt.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newt );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
    return ch;
}
#endif


int main()
{
    // 1. Set logger IP address
    const string ipAdress = "192.168.0.233";

    // 2. Create onlinestreaming interface
    int timeOutInMilliSec = 0;
    IBPNGStreaming* streaming = getBPNGStreamingInterface(timeOutInMilliSec, true);
    if (!streaming)
    {
        cout << "Failed to create pointer to OnlineStreamingInterface" << endl;
        return -1;
    }

    // 3. Create data streaming listener
    bool writeDataToFile = false;
    DataStreamingListener dataStreamListener(writeDataToFile);
    streaming->addStreamingListener((IBPNGStreamingListener*)&dataStreamListener);

    // 4. Disconnect interface from logger
    if (!streaming->connectLogger(ipAdress.c_str(), false))
    {
        releaseBPNGStreamingInterface(streaming);
        return -1;
    }

	// 5. Select channels you want to stream
    IChannelList* channelList = streaming->getChannelList();
    if(channelList->getSize() == 0)
    {
        cout << "Did not receive list of streaming channels from interface" << endl;
        releaseBPNGStreamingInterface(streaming);
        return -1;
    }

    IChannel* can2 = nullptr;
    for (unsigned int i = 0; i < channelList->getSize(); i++)
    {
        const IChannel* channel = channelList->getChannel(i);
        // e.g. only select channel CAN #1. Note that channel index always start at zero
        if (channel->getType() == CH_CAN && channel->getIndex() == 1)
        {
            can2 = (IChannel*) channel;
            streaming->setChannelFilter(channel->getType(), channel->getIndex(), true);
        }
    }

    cout << "Press any key to start streaming" << endl;
    _getch();
    time_t now = time(nullptr);
    cout << "Started at " << asctime(gmtime(&now)) << endl;

    // 6. Start streaming data
    if(!streaming->startDataStreaming())
    {
        dataStreamListener.onStreamingError(streaming->getLastError());
        releaseBPNGStreamingInterface(streaming);
        return -1;
    }

    cout << "Press any key to stop streaming" << endl;
    _getch();

    // 7. Stop streaming data
    if(!streaming->stopDataStreaming())
    {
        dataStreamListener.onStreamingError(streaming->getLastError());
        releaseBPNGStreamingInterface(streaming);
        return -1;
    }

    streaming->setChannelFilter(can2->getType(), can2->getIndex(), false);
    streaming->setChannelFilter(can2->getType(), can2->getIndex(), true);

    // 6. Start streaming data
    if (!streaming->startDataStreaming())
    {
        dataStreamListener.onStreamingError(streaming->getLastError());
        releaseBPNGStreamingInterface(streaming);
        return -1;
    }

    cout << "Press any key to stop streaming" << endl;
    _getch();

    // 7. Stop streaming data
    if (!streaming->stopDataStreaming())
    {
        dataStreamListener.onStreamingError(streaming->getLastError());
        releaseBPNGStreamingInterface(streaming);
        return -1;
    }

    // 9. Disconnect interface from logger
    streaming->disconnectLogger();
    releaseBPNGStreamingInterface(streaming);
    cout << "Streaming stopped, logger disconnected" << endl;
    cout << "Good bye (press any key)" << endl;
    _getch();
    return 0;
}
