//******************************************************************************
//******************************************************************************
/**
* @file    BPNGStreamingInterface.h
* @author  Desire Kaffo
* @date    30.01.2013
*
* @brief Interface class for the BPNG Online Data Streaming DLL
*
*/
// $Id$
//******************************************************************************
//******************************************************************************

#ifndef __DATASTREAMINGLISTENER
#define __DATASTREAMINGLISTENER

#include <iostream>
#include <string>
#include <vector>
#include <fstream>

#include "IBPNGStreaming.h"
#include "stdint.h"

// Example implemantation for a BPNGStreamingListener.
// There are two options how to handle the streamed data
// 1. write to file
// 2. output on cout
class DataStreamingListener : public IBPNGStreamingListener
{
public:
    DataStreamingListener(bool writeToFile = false, std::string path = "");
    ~DataStreamingListener();

    // Handle the message on reception time
    virtual void onStreamingMessage(const char* msg, const unsigned int length, const char* msgKey);
    virtual void onStreamingError(BPNGError err);
    virtual void onDataDiscarded(int numBytes, IChannelList* channels);
    virtual BOOL isASCIIRequested();
    virtual void onOnlineStreamingDisabled(uint32_t mbnr);

private:
    int m_streamASCII;
    bool m_writeToFile;
    std::ofstream m_outputFile;
};

#endif // __DATASTREAMINGLISTENER
